@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -42,9 +42,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <windows.h>
 //#define fileno _fileno
 //#define fcntl _fcntl
+#else
+#include <signal.h>
 #endif
 
-
 void rConsole::DoCenterDisplay(const tString &s,REAL timeout,REAL r,REAL g,REAL b){
     std::cout << tColoredString::RemoveColors(s) << '\n';
     DisplayAtNewline();
@@ -53,7 +54,22 @@ void rConsole::DoCenterDisplay(const tString &s,REAL timeout,REAL r,REAL g,REAL
 static int stdin_descriptor;
 static bool unblocked = false;
 
+static void sr_HandleSigCont( int signal )
+{
+    // con << "Continuing.\n";
+
+    // unblock stdin again
+    sr_Unblock_stdin();
+}
+
 void sr_Unblock_stdin(){
+#ifndef WIN32
+    if ( !unblocked )
+    {
+        signal( SIGCONT, &sr_HandleSigCont );
+    }
+#endif
+
     unblocked = true;
     stdin_descriptor=fileno(stdin);
 #ifndef WIN32
@@ -66,11 +82,15 @@ void sr_Unblock_stdin(){
 }
 
 
+
 #define MAXLINE 1000
 static char line_in[MAXLINE+2];
 static int currentIn=0;
 
 void sr_Read_stdin(){
+    // stdin commands are executed at owner level
+    tCurrentAccessLevel level( tAccessLevel_Owner, true );
+
     tConfItemBase::LoadPlayback( true );
 
     if ( !unblocked )
@@ -104,9 +124,9 @@ void sr_Read_stdin(){
                     if (key == 13 || currentIn>=MAXLINE-1){
                         line_in[currentIn]='\n';
                         line_in[currentIn+1]='\0';
-                        std::stringstream s(line_in);
+                        std::istringstream s(line_in);
                         WriteConsole(stdouthandle, "\n", 1, &written, NULL);
-                        tConfItemBase::LoadAll(s);
+                        tConfItemBase::LoadAll(s, true);
                         currentIn=0;
                     }
                     else
@@ -121,11 +141,12 @@ void sr_Read_stdin(){
 
 
 #else
-    while (read(stdin_descriptor,&line_in[currentIn],1)>0){
-        if (line_in[currentIn]=='\n' || currentIn>=MAXLINE-1){
+    while ( read(stdin_descriptor,&line_in[currentIn],1)>0){
+        if (line_in[currentIn]=='\n' || currentIn>=MAXLINE-1)
+        {
             line_in[currentIn+1]='\0';
-            std::stringstream s(line_in);
-            tConfItemBase::LoadAll(s);
+            std::istringstream s(line_in);
+            tConfItemBase::LoadAll(s, true);
             currentIn=0;
         }
         else
