@@ -47,8 +47,10 @@ gCamera::gCamera(eGrid *grid, rViewport *view,ePlayerNetID *p,ePlayer *lp,eCamMo
 
 
 gCamera::~gCamera(){
+#ifdef DEBUG
     int x;
     x =0;
+#endif
 }
 
 eCoord gCamera::CenterCycleDir(){
@@ -68,7 +70,7 @@ void gCamera::Timestep(REAL ts){
     eCamera::Timestep(ts);
     if (Center()!=lastCenter){
         if (!netPlayer || !netPlayer->Object() ||
-                netPlayer->Object()->ID()!=centerID){
+                netPlayer->Object()!=center){
             if (dynamic_cast<gCycle *>(Center())){
                 eNetGameObject *x=dynamic_cast<eNetGameObject *>(Center());
                 if (x){
@@ -77,7 +79,7 @@ void gCamera::Timestep(REAL ts){
                         con << tOutput("$camera_watching_ai");
                     else{
                         tOutput o;
-                        o.SetTemplateParameter(1, RemoveColors(p->name));
+                        o.SetTemplateParameter(1, p->GetName());
                         o << "$camera_watching_player";
                         con << o;
                     }
