@@ -87,10 +87,10 @@ static int sg_consoleHistoryMaxSize=10; // size of the console history
 static tSettingItem< int > sg_consoleHistoryMaxSizeConf("HISTORY_SIZE_CONSOLE",sg_consoleHistoryMaxSize);
 
 class ArmageTron_feature_menuitem: public uMenuItemSelection<int>{
-    void NewChoice(uSelectItem<bool> *){};
-    void NewChoice(char *,bool ){};
+    void NewChoice(uSelectItem<bool> *){}
+    void NewChoice(char *,bool ){}
 public:
-    ArmageTron_feature_menuitem(uMenu *m,char *tit,const char *help,int &targ)
+    ArmageTron_feature_menuitem(uMenu *m,char const * tit,char const * help,int &targ)
             :uMenuItemSelection<int>(m,tit,help,targ){
         uMenuItemSelection<int>::NewChoice(
             "$feature_disabled_text",
@@ -106,15 +106,15 @@ public:
             rFEAT_ON);
     }
 
-    ~ArmageTron_feature_menuitem(){};
+    ~ArmageTron_feature_menuitem(){}
 };
 
 
 class ArmageTron_texmode_menuitem: public uMenuItemSelection<int>{
-    void NewChoice(uSelectItem<bool> *){};
-    void NewChoice(char *,bool ){};
+    void NewChoice(uSelectItem<bool> *){}
+    void NewChoice(char *,bool ){}
 public:
-    ArmageTron_texmode_menuitem(uMenu *m,char *tit,int &targ,
+    ArmageTron_texmode_menuitem(uMenu *m,char const * tit,int &targ,
                                 bool font=false)
             :uMenuItemSelection<int>
     (m,tit,"$texture_menuitem_help",targ){
@@ -147,12 +147,9 @@ public:
     #endif
     }
 
-    ~ArmageTron_texmode_menuitem(){};
+    ~ArmageTron_texmode_menuitem(){}
 };
 
-static tConfItem<int>     la("LINE_ANTIALIAS",sr_lineAntialias);
-static tConfItem<int>     pa("POLY_ANTIALIAS",sr_polygonAntialias);
-static tConfItem<int>     pc("PERSP_CORRECT",sr_perspectiveCorrection);
 static tConfItem<bool>    ab("ALPHA_BLEND",sr_alphaBlend);
 static tConfItem<bool>    ss("SMOOTH_SHADING",sr_smoothShading);
 static tConfItem<bool>    to("TEXT_OUT",sr_textOut);
@@ -167,7 +164,10 @@ static tConfItem<bool> ls("LOWER_SKY",sr_lowerSky);
 static tConfItem<bool> wos("SKY_WOBBLE",sr_skyWobble);
 static tConfItem<bool> ip("INFINITY_PLANE",sr_infinityPlane);
 
+extern bool sg_axesIndicator;
+
 static tConfItem<bool> lm("LAG_O_METER",sr_laggometer);
+static tConfItem<bool> ai("AXES_INDICATOR",sg_axesIndicator);
 static tConfItem<bool> po("PREDICT_OBJECTS",sr_predictObjects);
 static tConfItem<bool> t32("TEXTURES_HI",sr_texturesTruecolor);
 
@@ -234,12 +234,16 @@ public:
         else
         {
             // add custom resolution
-            NewChoice( ArmageTron_Custom );//, "$screen_custom_text","$screen_custom_help" );
+            NewChoice( ArmageTron_Custom );
+
+            // add desktop resolution
+            if ( sr_DesktopScreensizeSupported() && !addFixed )
+                NewChoice( ArmageTron_Desktop );
 
             // the maximal allowed screen size
             rScreenSize maxSize(0,0);
 
-            // fill in available modes (avoid dublicates)
+            // fill in available modes (avoid duplicates)
             for(i=0;modes[i];++i)
             {
                 // add mode (if it's new)
@@ -254,7 +258,7 @@ public:
             // add fixed resolutions (as window sizes)
             if ( addFixed )
             {
-                for ( i = ArmageTron_Custom; i>=0; --i )
+                for ( i = ArmageTron_Custom; i>=ArmageTron_Min; --i )
                 {
                     rScreenSize size( static_cast< rResolution >(i) );
 
@@ -271,7 +275,11 @@ public:
                 rScreenSize const & size = *iter;
 
                 std::stringstream s;
-                s << size.width << " x " << size.height;
+                if ( size.width + size.height > 0 )
+                    s << size.width << " x " << size.height;
+                else
+                    s << tOutput("$screen_size_desktop");
+
                 res_men.NewChoice( s.str().c_str(), help, size );
             }
 
@@ -418,17 +426,6 @@ static uSelectEntry<int> mfdd(mfd,"$detail_floor_2tex_text",
                               "$detail_floor_2tex_help",
                               rFLOOR_TWOTEXTURE);
 
-
-static ArmageTron_feature_menuitem pam
-(&screen_menu_detail,"$detail_polyantialias_text",
- "$detail_polyantialias_help",
- sr_polygonAntialias);
-
-static ArmageTron_feature_menuitem lam
-(&screen_menu_detail,"$detail_lineantialias_text",
- "$detail_lineantialias_help",
- sr_lineAntialias);
-
 static uMenuItemToggle  abm
 (&screen_menu_detail,"$detail_alpha_text",
  "$detail_alpha_help",
@@ -439,11 +436,6 @@ static uMenuItemToggle  ssm
  "$detail_smooth_help",
  sr_smoothShading);
 
-static ArmageTron_feature_menuitem pcm
-(&screen_menu_detail,"$detail_persp_text",
- "$detail_persp_help",
- sr_perspectiveCorrection);
-
 extern bool crash_sparks;		// from gCycle.cpp
 extern bool white_sparks;		// from gSparks.cpp
 static tConfItem<bool> cs2("SPARKS",crash_sparks);
@@ -519,13 +511,12 @@ static uMenuItemToggle cs
  "$pref_sparks_help",
  crash_sparks);
 
-static uMenuItemToggle dl
+static uMenuItemSelection<rDisplayListUsage> dl
 (&screen_menu_tweaks,"$tweaks_displaylists_text",
  "$tweaks_displaylists_help", sr_useDisplayLists);
-
-static uMenuItemToggle zt
-(&screen_menu_tweaks,"$tweaks_ztrick_text",
- "$tweaks_ztrick_help",sr_ZTrick);
+static uSelectEntry<rDisplayListUsage> dl_off(dl,"$tweaks_displaylists_off_text","$tweaks_displaylists_off_help",rDisplayList_Off);
+static uSelectEntry<rDisplayListUsage> dl_cac(dl,"$tweaks_displaylists_cac_text","$tweaks_displaylists_cac_help",rDisplayList_CAC);
+static uSelectEntry<rDisplayListUsage> dl_cae(dl,"$tweaks_displaylists_cae_text","$tweaks_displaylists_cae_help",rDisplayList_CAE);
 
 static uMenuItemToggle infp
 (&screen_menu_tweaks,"$tweaks_infinity_text",
@@ -555,7 +546,9 @@ static uSelectEntry<rSysDep::rSwapMode> swapMode_80Hz(swapMode,"$swapmode_80hz_t
 static uSelectEntry<rSysDep::rSwapMode> swapMode_60Hz(swapMode,"$swapmode_60hz_text","$swapmode_60hz_help",rSysDep::rSwap_60Hz);
 */
 
-static tConfItem<int> swapModeCI("SWAP_MODE", reinterpret_cast< int & >( rSysDep::swapMode_ ) );
+tCONFIG_ENUM( rSysDep::rSwapMode );
+
+static tConfItem< rSysDep::rSwapMode > swapModeCI("SWAP_MODE", rSysDep::swapMode_ );
 
 static tConfItem<REAL> sgs("SPEED_GAUGE_SIZE",subby_SpeedGaugeSize);
 static tConfItem<REAL> sgx("SPEED_GAUGE_LOCX",subby_SpeedGaugeLocX);
@@ -659,18 +652,12 @@ public:
 
             con << tColoredString::ColorString(.5,.5,1) << " > " << *content << '\n';
 
-            if ( tRecorder::IsPlayingBack() )
-            {
-                // the command was also recorded; better play it back,
-                // or the playback gets out of sync
-                tConfItemBase::LoadPlayback( false );
-            }
-            else
-            {
-                // pass the console command to the configuration system
-                std::stringstream s(&((*content)[0]));
-                tConfItemBase::LoadAll(s);
-            }
+            // direct commands are executed at owner level
+            tCurrentAccessLevel level( tAccessLevel_Owner, true );
+
+            // pass the console command to the configuration system
+            std::stringstream s(&((*content)[0]));
+            tConfItemBase::LoadAll( s, false );
 
             MyMenu()->Exit();
             return true;
@@ -833,7 +820,7 @@ public:
         m->RequestSpaceBelow(.2);
     }
 
-    ~ArmageTron_color_menuitem(){};
+    ~ArmageTron_color_menuitem(){}
 
     virtual REAL SpaceRight(){return .2;}
 
@@ -878,6 +865,7 @@ void sg_PlayerMenu(int Player){
     uMenu camera_menu("$player_camera_text");
     uMenu chat_menu("$player_chat_text");
     //  name.Clear();
+    chat_menu.SetCenter(-.5);
 
     uMenuItemString *ic[MAX_INSTANT_CHAT];
 
@@ -896,6 +884,21 @@ void sg_PlayerMenu(int Player){
                p->instantChatString[i], se_SpamMaxLen);
     }
 
+    uMenuItemToggle al
+    (&playerMenu,"$player_autologin_text",
+     "$player_autologin_help",
+     p->autoLogin);
+
+    uMenuItemString gid(&playerMenu,
+                      "$player_global_id_text",
+                      "$player_global_id_help",
+                      p->globalID, 400);
+    gid.SetColorMode( rTextField::COLOR_IGNORE );
+
+    uMenuItemToggle st
+    (&playerMenu,"$player_stealth_text",
+     "$player_stealth_help",
+     p->stealth);
 
     uMenuItemToggle sp
     (&playerMenu,"$player_spectator_text",
@@ -1035,7 +1038,10 @@ void sg_PlayerMenu(int Player){
     // request network synchronisation if the server can handle it
     static nVersionFeature inGameRenames( 5 );
     if ( inGameRenames.Supported() )
+    {
         ePlayerNetID::Update();
+        ePlayer::SendAuthNames();
+    }
 
     /*
     for (i=MAX_PLAYERS-1; i>=0; i--)
@@ -1150,11 +1156,20 @@ static bool con_func(REAL x){
 
 static bool toggle_fullscreen_func( REAL x )
 {
-    if ( x > 0 )
+#ifndef DEDICATED
+#ifdef DEBUG
+    // don't toggle fullscreen while playing back in debug mode, that's annoying
+    if ( tRecorder::IsPlayingBack() )
+        return true;
+#endif
+
+    // only do anything if the application is active (work around odd bug)
+    if ( x > 0 && ( SDL_GetAppState() & SDL_APPACTIVE ) )
     {
         currentScreensetting.fullscreen = !currentScreensetting.fullscreen;
         sr_ReinitDisplay();
     }
+#endif
 
     return true;
 }
