@@ -21,7 +21,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -46,10 +46,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rSDL.h"
 #endif
 
+#include <vector>
+
 FUNCPTR  uMenu::idle(NULL);
 
 bool uMenu::wrap=true;
-bool uMenu::quickexit=false;
+uMenu::QuickExit uMenu::quickexit=uMenu::QuickExit_Off;
 bool uMenu::exitToMain=false;
 
 // *****************************************************
@@ -62,7 +64,7 @@ uMenu::uMenu(const char *t="",bool exit_item)
     menuTop=.7;
     menuBot=-.7;
     yOffset=0;
-    selected = -1;
+    selected = 10000000;
 }
 #endif
 
@@ -73,17 +75,17 @@ uMenu::uMenu(const tOutput &t,bool exit_item)
     menuTop=.7;
     menuBot=-.7;
     yOffset=0;
-    selected = -1;
+    selected = 100000000;
 }
 
 uMenu::~uMenu(){
-    for(int i=items.Len()-1;i>=0;i--)
+    for (int i=items.Len()-1;i>=0;i--)
         delete items[i];
 }
 
 void uMenu::ReverseItems(){
-    tList<uMenuItem> dummy = items;
-    items.SetLen(0);
+    tList<uMenuItem> dummy;
+    dummy.Swap( items );
 
     for (int i=dummy.Len()-1; i>=0; i--){
         uMenuItem *x = dummy[i];
@@ -166,7 +168,7 @@ void uMenu::OnEnter(){
     REAL lastt=0;
     REAL ts=0;
 
-#ifndef DEDICATED  
+#ifndef DEDICATED
     lastkey=tSysTimeFloat();
     static const REAL timeout=3;
 #endif
@@ -175,27 +177,35 @@ void uMenu::OnEnter(){
         st_DoToDo();
         tAdvanceFrame();
 
-        if (selected < 0 || selected >= items.Len())
-            selected = items.Len()-1;
-
         ts=tSysTimeFloat()-lastt;
         lastt=tSysTimeFloat();
         if (ts>.2) ts=.2;
 
         menuentries=items.Len();
 
+        // clamp cursor
+        if (selected < 0 )
+            selected = 0;
+        if ( selected >= items.Len())
+            selected = items.Len()-1;
 
 #ifndef DEDICATED
         {
             SDL_Event tEvent;
             uInputProcessGuard inputProcessGuard;
-            while(su_GetSDLInput(tEvent))
+            while (su_GetSDLInput(tEvent))
             {
                 REAL entertime = tSysTimeFloat();
 
                 switch (tEvent.type)
                 {
                 case SDL_KEYDOWN:
+                    if ( tEvent.key.keysym.sym == SDLK_UNKNOWN )
+                    {
+                        // don't repeat unknown syms. They come from multi-key compositions and
+                        // don't send keyup events when released.
+                        break;
+                    }
                     repeat = true;
                     memcpy( &tEventRepeat, &tEvent, sizeof( SDL_Event ) );
                     nextrepeat = tSysTimeFloat() + repeatdelay;
@@ -223,6 +233,15 @@ void uMenu::OnEnter(){
                 nextrepeat = tSysTimeFloat() + repeatrate;
             }
         }
+
+        // we're about to render, last chance to make changes to the menu
+        OnRender();
+
+        // clamp cursor
+        if (selected < 0 )
+            selected = 0;
+        if ( selected >= items.Len())
+            selected = items.Len()-1;
 #endif
         // quit shortcut
         if ( quickexit )
@@ -257,7 +276,6 @@ void uMenu::OnEnter(){
             yOffset+=menuTop-smallborder-YPos(menuentries-1);
 
 #ifndef DEDICATED
-        rSysDep::ClearGL();
         sr_ResetRenderState(true);
         items[selected]->RenderBackground();
 
@@ -268,14 +286,14 @@ void uMenu::OnEnter(){
         if (sr_glOut && !exitFlag && !quickexit){
             items[selected]->Render(center,YPos(selected),1,true);
 
-            for(int i=items.Len()-1;i>=0;i--)
-                if(i!=selected){
+            for (int i=items.Len()-1;i>=0;i--)
+                if (i!=selected){
                     REAL y=YPos(i);
                     REAL alpha=1;
                     const REAL b=.1;
-                    if(y<menuBot+b)
+                    if (y<menuBot+b)
                         alpha=(y-menuBot)/b;
-                    if(y>menuTop-b)
+                    if (y>menuTop-b)
                         alpha=(menuTop-y)/b;
                     if (y>menuBot && y<menuTop)
                     {
@@ -318,12 +336,14 @@ void uMenu::OnEnter(){
         }
         else
 #endif
-        {
-            tDelay( 100000 );
-        }
+            if ( !sr_glOut )
+            {
+                tDelay( 10000 );
+            }
 
 #ifndef DEDICATED
         rSysDep::SwapGL();
+        rSysDep::ClearGL();
 #endif
     }
 
@@ -340,90 +360,94 @@ void uMenu::HandleEvent( SDL_Event event )
     {
         switch (event.type){
         case SDL_KEYDOWN:
-            {
-                if (!disphelp)
-                    lastkey=tSysTimeFloat();
-                switch (event.key.keysym.sym){
+        {
+            if (!disphelp)
+                lastkey=tSysTimeFloat();
+            switch (event.key.keysym.sym){
 
-                case(SDLK_ESCAPE):
-                                repeat = false;
-                    lastkey=tSysTimeFloat();
-                    Exit();
-                    break;
+            case(SDLK_ESCAPE):
+                            repeat = false;
+                lastkey=tSysTimeFloat();
+                Exit();
+                break;
 
-                case(SDLK_UP):
-                                lastkey=tSysTimeFloat();
-                    selected++;
-                    if (selected>=items.Len())
-                        if (wrap)
-                            selected=0;
-                        else
-                            selected=items.Len()-1;
-                    break;
+            case(SDLK_UP):
+                            lastkey=tSysTimeFloat();
+                selected++;
+                if (selected>=items.Len())
+                {
+                    if (wrap)
+                        selected=0;
+                    else
+                        selected=items.Len()-1;
+                }
+                break;
 
-                case(SDLK_DOWN):
-                                lastkey=tSysTimeFloat();
-                    selected--;
-                    if (selected<0)
-                        if(wrap)
-                            selected=items.Len()-1;
-                        else
-                            selected=0;
+            case(SDLK_DOWN):
+                            lastkey=tSysTimeFloat();
+                selected--;
+                if (selected<0)
+                {
+                    if (wrap)
+                        selected=items.Len()-1;
+                    else
+                        selected=0;
+                }
 
-                    break;
+                break;
 
-                case(SDLK_LEFT):
-                                items[selected]->LeftRight(-1);
-                    break;
-                case(SDLK_RIGHT):
-                                items[selected]->LeftRight(1);
-                    break;
+            case(SDLK_LEFT):
+                            items[selected]->LeftRight(-1);
+                break;
+            case(SDLK_RIGHT):
+                            items[selected]->LeftRight(1);
+                break;
 
-                case(SDLK_SPACE):
-                            case(SDLK_KP_ENTER):
-                                case(SDLK_RETURN):
-                                        repeat = false;
-                    try
-            {
-                        su_inMenu = false;
-                        items[selected]->Enter();
-                    }
-                    catch (tException const & e)
-                    {
-                        uMenu::SetIdle(NULL);
+            case(SDLK_SPACE):
+                        case(SDLK_KP_ENTER):
+                            case(SDLK_RETURN):
+                                    repeat = false;
+                try
+        {
+                    su_inMenu = false;
+                    items[selected]->Enter();
+                }
+                catch (tException const & e)
+                {
+                    uMenu::SetIdle(NULL);
 
-                        // inform user of generic errors
-                        tConsole::Message( e.GetName(), e.GetDescription(), 20 );
-                    }
+                    // inform user of generic errors
+                    tConsole::Message( e.GetName(), e.GetDescription(), 20 );
+                }
 #ifdef _MSC_VER
 #pragma warning ( disable : 4286 )
-                    // GRR. Visual C++ dones not handle generic exceptions with the above general statement.
-                    // A specialized version is needed. The best part: it warns about the code below being redundant.
-                    catch( tGenericException const & e )
+                // GRR. Visual C++ dones not handle generic exceptions with the above general statement.
+                // A specialized version is needed. The best part: it warns about the code below being redundant.
+                catch ( tGenericException const & e )
+                {
+                    try
                     {
-                        try
-                        {
-                            tConsole::Message( e.GetName(), e.GetDescription(), 20 );
-                        }
-                        catch(...)
-                        {
-                        }
+                        tConsole::Message( e.GetName(), e.GetDescription(), 20 );
                     }
+                    catch (...)
+                    {
+                    }
+                }
 #endif
 
-                    su_inMenu = true;
+                su_inMenu = true;
 
-                    repeat = false;
-                    lastkey=tSysTimeFloat();
-                    break;
+                repeat = false;
+                lastkey=tSysTimeFloat();
+                break;
 
-                default:
-                    // let the input subsystem handle events for later processing
-                    su_HandleEvent( event, true );
-                    break;
-                }
+            default:
+                // let the input subsystem handle events for later processing
+                su_HandleEvent( event, true );
+                break;
             }
-            break;
+        }
+        break;
         default:
             // let the input subsystem handle events for later processing
             su_HandleEvent( event, true );
@@ -446,7 +470,7 @@ void uMenu::GenericBackground(){
             // throw tGenericException("test"); // (test exception throw to see if error handling works right)
             (*idle)();
         }
-        catch( ... )
+        catch ( ... )
         {
             // the idle background function is broken. Disable it and rethrow.
             idle = 0;
@@ -467,6 +491,11 @@ void uMenu::OnExit(){
     exitFlag=1;
 }
 
+//! called every frame before the menu is rendered
+void uMenu::OnRender()
+{
+}
+
 // *****************************************************
 
 // *******************************************************************************************
@@ -485,7 +514,7 @@ void uMenuItem::SetColor( bool selected, REAL alpha )
     //   rTextField::SetBlendColor( tColor(.8+.2*sin(time),.3-.1*sin(time),.3-.1*sin(time),alpha) );
     rTextField::SetDefaultColor( tColor(1,1,1,alpha) );
 
-    if(selected)
+    if (selected)
     {
         REAL time=tSysTimeFloat()*10;
         REAL intensity = 1+.3*sin(time);
@@ -496,7 +525,7 @@ void uMenuItem::SetColor( bool selected, REAL alpha )
 
 void uMenuItem::DisplayText(REAL x,REAL y,const char *text,
                             bool selected,REAL alpha,
-                            int center,int c,int cp){
+                            int center,int c,int cp, rTextField::ColorMode colorMode ){
 #ifndef DEDICATED
     if (sr_glOut){
         SetColor( selected, alpha );
@@ -504,6 +533,9 @@ void uMenuItem::DisplayText(REAL x,REAL y,const char *text,
         REAL tw = text_width;
         REAL th = text_height;
 
+        
+#if 0
+        // the function that is called takes care of that
         REAL availw = 1.9f;
         if (center < 0) availw = (.9f-x);
         if (center > 0) availw = (x + .9f);
@@ -514,8 +546,9 @@ void uMenuItem::DisplayText(REAL x,REAL y,const char *text,
             th *= availw/(len * tw);
             tw  = availw/len;
         }
+#endif
 
-        ::DisplayText(x,y,tw,th,text,center,c,cp);
+        ::DisplayText(x,y,tw,th,text,center,c,cp, colorMode );
     }
 #endif
 }
@@ -595,7 +628,7 @@ uMenuItemInt::uMenuItemInt
 (uMenu *m,const char *tit,const char *help,int &targ,
  int mi,int ma,int step)
         :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
-Step(step){
+        Step(step){
     if (target<Min) target=Min;
     if (target>Max) target=Max;
 }
@@ -605,7 +638,7 @@ uMenuItemInt::uMenuItemInt
 (uMenu *m,const tOutput &tit,const tOutput &help,int &targ,
  int mi,int ma,int step)
         :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
-Step(step){
+        Step(step){
     if (target<Min) target=Min;
     if (target>Max) target=Max;
 }
@@ -626,6 +659,46 @@ void uMenuItemInt::Render(REAL x,REAL y,REAL alpha,
     DisplayText(x+.02,y,s,selected,alpha,-1);
 }
 
+// *****************************************
+//               Float Choose
+// *****************************************
+
+#ifdef SLOPPYLOCALE
+uMenuItemReal::uMenuItemReal
+(uMenu *m,const char *tit,const char *help,REAL &targ,
+ REAL mi,REAL ma,REAL step)
+        :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
+        Step(step){
+    if (target<Min) target=Min;
+    if (target>Max) target=Max;
+}
+#endif
+
+uMenuItemReal::uMenuItemReal
+(uMenu *m,const tOutput &tit,const tOutput &help,REAL &targ,
+ REAL mi,REAL ma,REAL step)
+        :uMenuItem(m,help),title(tit),target(targ),Min(mi),Max(ma),
+        Step(step){
+    if (target<Min) target=Min;
+    if (target>Max) target=Max;
+}
+
+
+void uMenuItemReal::LeftRight(int dir){
+    target+=dir*Step;
+    if (target<Min) target=Min;
+    if (target>Max) target=Max;
+}
+
+void uMenuItemReal::Render(REAL x,REAL y,REAL alpha,
+                          bool selected){
+    DisplayText(x-.02,y,title,selected,alpha,1);
+
+    tString s;
+    s << target;
+    DisplayText(x+.02,y,s,selected,alpha,-1);
+}
+
 
 // *****************************************************
 
@@ -639,6 +712,7 @@ uMenuItemString::uMenuItemString(uMenu *M,
     if (len==0 || (*content)(len-1)!=0)
         (*content)[len]=0;
     cursorPos=content->Len()-1;
+    colorMode_ = rTextField::COLOR_SHOW;
 }
 
 void uMenuItemString::Render(REAL x,REAL y,
@@ -653,8 +727,13 @@ void uMenuItemString::Render(REAL x,REAL y,
         if (counter & 32) cmode=2;
     }
 
+    // unslected items with COLOR_SHOW should be rendered with COLOR_USE
+    rTextField::ColorMode colorMode = colorMode_;
+    if ( colorMode == rTextField::COLOR_SHOW && !selected )
+        colorMode = rTextField::COLOR_USE;
+
     DisplayText(x-.02,y,description,selected,alpha,1);
-    DisplayText(x+.02,y,&((*content)[0]),selected,alpha,-1,cmode,cursorPos);
+    DisplayText(x+.02,y,&((*content)[0]),selected,alpha,-1,cmode,cursorPos, colorMode );
 #endif
 }
 
@@ -785,7 +864,7 @@ bool uMenuItemString::Event(SDL_Event &e){
             // insert character if there is room
             if (content->Len() < maxLength_)
             {
-                for(int i=content->Len()-1;i>=cursorPos;i--)
+                for (int i=content->Len()-1;i>=cursorPos;i--)
                     (*content)[i+1]=(*content)[i];
 
                 // guarantee proper null termination
@@ -799,11 +878,11 @@ bool uMenuItemString::Event(SDL_Event &e){
         }
     }
 
-    if(cursorPos<0)    cursorPos=0;
-    if(cursorPos > content->Len()-1) cursorPos=content->Len()-1;
+    if (cursorPos<0)    cursorPos=0;
+    if (cursorPos > content->Len()-1) cursorPos=content->Len()-1;
 
     return ret;
-#else  
+#else
     return false;
 #endif
 }
@@ -895,7 +974,7 @@ bool uMenuItemStringWithHistory::Event(SDL_Event &e)
 uMenuItemSubmenu::uMenuItemSubmenu(uMenu *M,
                                    uMenu *s,
                                    const tOutput& help)
-    :uMenuItem(M,help),submenu(s){}
+        :uMenuItem(M,help),submenu(s){}
 
 
 void uMenuItemSubmenu::Render(REAL x,REAL y,REAL alpha,bool selected){
@@ -997,7 +1076,7 @@ void uMenuItemFileSelection::AddFile( const char *fileName, const char *filePath
 static tCallback *enter_anchor=NULL,*leave_anchor=NULL, *background_anchor=NULL;
 
 uCallbackMenuEnter::uCallbackMenuEnter(VOIDFUNC *f)
-    :tCallback(enter_anchor,f){}
+        :tCallback(enter_anchor,f){}
 
 void uCallbackMenuEnter::MenuEnter(){
     Exec(enter_anchor);
@@ -1017,9 +1096,39 @@ void uCallbackMenuBackground::MenuBackground(){
     Exec(background_anchor);
 }
 
+// poll input, return true if ESC was pressed
+bool uMenu::IdleInput()
+{
+#ifndef DEDICATED
+    SDL_Event event;
+    uInputProcessGuard inputProcessGuard;
+    while (su_GetSDLInput(event))
+    {   
+        switch (event.type)
+        {
+        case SDL_KEYDOWN:
+            switch (event.key.keysym.sym)
+            {
+            case(SDLK_ESCAPE):
+                repeat = false;
+                lastkey=tSysTimeFloat();
+                return true;
+                break;
+            default:
+                break;
+            }   
+        default:
+            break;
+        }
+    }   
+#endif
 
+    return false;
+}
 
-void uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL to){
+// return value: false only if the user pressed ESC
+bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL to){
+    bool ret = true;
 #ifdef DEDICATED
     con << message << ":\n";
     con << interpretation << '\n';
@@ -1053,14 +1162,45 @@ void uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
     // catch some keyboard input
     {
         uInputProcessGuard inputProcessGuard;
-        while(su_GetSDLInput(tEvent));
+        while (su_GetSDLInput(tEvent)) ;
     }
 
     {
         uInputProcessGuard inputProcessGuard;
-        while(  !quickexit && ( !su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
-                (to < 0 || tSysTimeFloat() < timeout)){
 
+        unsigned offset = 0; //amount of scrolling taking place
+        //convert to an array for scrolling
+        tString interpretationString;
+        interpretationString << interpretation << "\n";
+        std::vector<tString> lines;
+        int lastNewline = 0;
+        for (int i = 0; i < interpretationString.Len() - 1; ++i) {
+            if (interpretationString[i] == '\n' && i != 0) {
+                lines.push_back(interpretationString.SubStr(lastNewline, i - lastNewline));
+                lastNewline = i + 1;
+            }
+        }
+        while (  !quickexit &&
+                 (to < 0 || tSysTimeFloat() < timeout)){
+            //while(  !quickexit && ( !su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
+            //        (to < 0 || tSysTimeFloat() < timeout)){
+            if ( su_GetSDLInput(tEvent) && tEvent.type==SDL_KEYDOWN) {
+                switch (tEvent.key.keysym.sym) {
+                case SDLK_UP:
+                    if (offset > 0)
+                        offset -= 1;
+                    continue;
+                case SDLK_DOWN:
+                    offset += 1;
+                    continue;
+                case SDLK_ESCAPE:
+                    ret = false;
+                    break;
+                default:
+                    break;
+                }
+                break;
+            }
             if ( sr_glOut )
             {
                 sr_ResetRenderState(true);
@@ -1090,14 +1230,15 @@ void uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
                 w = 16/640.0;
                 h = 32/480.0;
 
+                if (offset >= lines.size()) offset = lines.size() - 1;
                 {
                     rTextField c(-.8,.6, w, h);
 
-                    c << interpretation;
+                    for (unsigned i = offset; i < lines.size(); ++i)
+                        c << lines[i] << "\n";
                 }
-
-                rSysDep::SwapGL();
             }
+            rSysDep::SwapGL();
             tAdvanceFrame();
         }
     }
@@ -1105,7 +1246,7 @@ void uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
     // catch some keyboard input
     {
         uInputProcessGuard inputProcessGuard;
-        while (su_GetSDLInput(tEvent));
+        while (su_GetSDLInput(tEvent)) ;
     }
 
     uMenu::SetIdle(idle_back);
@@ -1115,5 +1256,7 @@ void uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
 
     sr_textOut = textOutBack;
 #endif
+
+    return ret;
 }
 
